import json
from VIStk.Structures.VINFO import *
from VIStk.Structures.screen import *
from VIStk.Objects._Root import *

class Project(VINFO):
    """VIS Project Object
    """
    def __init__(self):
        """Initializes or load a VIS project
        """
        super().__init__()
        with open(self.p_sinfo,"r") as f:
            info = json.load(f)
            self.name = list(info.keys())[0]

            for screen in list(info[self.name]["Screens"].keys()):
                scr = Screen(screen,
                             info[self.name]["Screens"][screen]["script"],
                             info[self.name]["Screens"][screen]["release"],
                             info[self.name]["Screens"][screen].get("icon"),
                             exists=True)
                self.screenlist.append(scr)
            self.d_icon = info[self.name]["defaults"]["icon"]

            self.dist_location:str = info[self.name]["release_info"]["location"]
            self.hidden_imports:list[str] = info[self.name]["release_info"]["hidden_imports"]
    
    def newScreen(self,screen:str) -> int:
        """Creates a new screen with some prompting

        Returns:
            0 Failed
            1 Success
        """
        #Check for valid filename  
        if not validName(screen):
            return 0
        
        with open(self.p_sinfo,"r") as f:
            info = json.load(f) #Load info

        name = self.title
        if info[name]["Screens"].get(screen) == None: #If Screen does not exist in VINFO
            while True: #ensures a valid name is used for script
                match input(f"Should python script use name {screen}.py? "):
                    case "Yes" | "yes" | "Y" | "y":
                        script = screen + ".py"
                        break
                    case _:
                        script = input("Enter the name for the script file: ").strip(".py")+".py"
                        if validName(script):
                            break

            match input("Should this screen have its own .exe?: "):
                case "Yes" | "yes" | "Y" | "y":
                    release = True
                case _:
                    release = False
            ictf =input("What is the icon for this screen (or none)?: ")
            icon = ictf.strip(".ico") if ".ICO" in ictf.upper() else None
            desc = input("Write a description for this screen: ")
            self.screenlist.append(Screen(screen,script,release,icon,False,desc))

            return 1
        else:
            print(f"Information for {screen} already in project.")
            return 1

    def hasScreen(self,screen:str) -> bool:
        """Checks if the project has the correct screen
        """
        for i in self.screenlist:
            if i.name == screen:
                return True
        return False
    
    def getScreen(self,screen:str) -> Screen:
        """Returns a screen object by its name
        """
        for i in self.screenlist:
            if i.name == screen:
                return i
        return None

    def verScreen(self,screen:str) -> Screen:
        """Verifies a screen exists and returns it

        Returns:
            screen (Screen): Verified screen
        """
        if not self.hasScreen(screen):
            self.newScreen(screen)
        scr = self.getScreen(screen)
        return scr

    def saveState(self)->None:
        """Saves the current project state to project.json | WILL FINISH THIS LATER"""
        with open(self.p_sinfo, "w") as f:
            info = {}
            info[self.name] = {}
            info[self.name]["Screens"] = {}


import sys
import os
import zipfile
from importlib import metadata
from VIStk.Structures import *

inp = sys.argv

#Copied from source https://stackoverflow.com/a/75246706
def unzip_without_overwrite(src_path, dst_dir):
    with zipfile.ZipFile(src_path, "r") as zf:
        for member in zf.infolist():
            file_path = os.path.join(dst_dir, member.filename)
            if not os.path.exists(file_path):
                zf.extract(member, dst_dir)

def __main__():
    match inp[1]:
        case "-v"|"-V"|"-Version"|"-version":
            print(f"VIS Version {metadata.version('VIStk')}")
            
        case "new"|"New"|"N"|"n":#Create a new VIS project
            project = VINFO()

        case "add" | "Add" | "a" | "A":
            project = Project()
            match inp[2]:
                case "screen" | "Screen" | "s" | "S":
                    if not inp[3] == None:
                        screen = project.verScreen(inp[3])
                        if len(inp) >= 5:
                            match inp[4]:
                                case "menu" | "Menu" | "m" | "M":
                                    screen.addMenu(inp[5])
                                case "elements" | "Elements" | "e" | "E":
                                    for i in inp[5].split("-"):
                                        screen.addElement(i)
                                    screen.stitch()
                        else:
                            project.newScreen(inp[3])

        case "stitch" | "Stitch" | "s" | "S":
            project = Project()
            screen = project.getScreen(inp[2])
            if not screen == None:
                screen.stitch()
            else:
                print("Screen does not exist")

        case "release" | "Release" | "r" | "R":
            flag:str=""
            type:str=""
            note:str=""
            argstart = 2

            if len(inp) >= 3:
                if inp[2] in ["Screen", "screen","S","s"]:
                    argstart = 4
                    screen = findScreen(inp[3])
                    if not screen is None:
                        screen.isolate()

                    else:
                        print(f"Cannot Locate Screen: \"{inp[3]}\"")
                        return None

                args = inp[argstart:]
                i=0
                while i < len(args):
                    if "-" == args[i][0]:
                        match args[i][1:]:
                            case "Flag" | "flag" | "F" | "f":
                                flag = args[i+1]
                                i += 2
                            case "Type" | "type" | "T" | "t":
                                type = args[i+1]
                                i += 2
                            case "Note" | "note" | "N" | "n":
                                note = args[i+1]
                                i += 2
                            case _:
                                print(f"Unknown Argument \"{args[i]}\"")
                                return None

            rel = Release(flag,type,note)
            rel.release()
            rel.restoreAll()


import os
import json
import zipfile
import shutil
import subprocess
import sys
import VIStk

VISROOT = VIStk.__file__.replace("__init__.pyc","").replace("__init__.py","")

#Copied from source
#https://stackoverflow.com/a/75246706
def unzip_without_overwrite(src_path, dst_dir):
    with zipfile.ZipFile(src_path, "r") as zf:
        for member in zf.infolist():
            file_path = os.path.join(dst_dir, member.filename)
            if not os.path.exists(file_path):
                zf.extract(member, dst_dir)

def getPath()->str:
    """Searches for .VIS folder
    """
    wd = os.getcwd().replace("\\","/").split("/")
    for i in range(len(wd),0,-1):
        if os.path.exists("/".join(wd[:i])+"/.VIS/"):
            return "/".join(wd[:i])
    else:
        return None

        
def validName(name:str):
    """Checks if provided path is a valid filename
    """
    if " " in name:
        print("Cannot have spaces in file name.")
        return False
    if "/" in name or "\\" in name:
        print("Cannot have filepath deliminator in file name.")
        return False
    if "<" in name or ">" in name or ":" in name or '"' in name or "|" in name or "?" in name or "*" in name:
        print('Invlaid ASCII characters for windows file creation, please remove all <>:"|?* from file name.')
        return False
    if name.split(".")[0] in ["CON","PRN","AUX","NUL","COM1","COM2","COM3","COM4","COM5","COM6","COM7","COM8","COM9","LPT1","LPT2","LPT3","LPT4","LPT5","LPT6","LPT7","LPT8","LPT9"]:
        print(f"Filename {name} reserved by OS.")
        return False
    if "" == name:
        print("Must provide a name for file.")
        return False
    else:
        return True
    

class VINFO():
    """Overarching control structure within the /.VIS/ project folder"""
    def __init__(self):
        """Creates an overarching control stricture within the /.VIS/ project folder
        """
        if getPath() == None:
            wd = os.getcwd()
            os.mkdir(wd+"\\.VIS")

            unzip_without_overwrite(VISROOT+"Form.zip",wd)
            print(f"Copied structure to {wd}")

            shutil.copytree(VISROOT+"Templates",wd+"/.VIS/Templates",dirs_exist_ok=True)
            print(f"Loaded default templates into {wd}/.VIS/Templates/")

           
            #DO NOT MESS WITH THE TEMPLATE HEADERS

            title = input("Enter a name for the VIS project: ")
            self.title:str = title #Name of the project
            info = {}
            info[self.title] = {}
            info[self.title]["Screens"]={}
            info[self.title]["defaults"]={}
            info[self.title]["defaults"]["icon"]="VIS"#default icon
            self.d_icon = "VIS"
            info[self.title]["metadata"]={}
            comp = input("What company is this for(or none)? ")
            if not comp in ["none","None"]:
                info[self.title]["metadata"]["company"] = comp
                self.company = comp
            else:
                info[self.title]["metadata"]["company"] = None
                self.company = None

            version = input("What is the initial version for the project (0.0.1 default): ")
            vers = version.split(".")
            if len(vers)==3:
                if vers[0].isnumeric() and vers[1].isnumeric() and vers[2].isnumeric():
                    self.version = version
                else:
                    self.version = "0.0.1"
            else:
                self.version = "0.0.1"
            info[self.title]["metadata"]["version"] = self.version

            with open(wd+"/.VIS/project.json","w") as f:
                json.dump(info,f,indent=4)
            print(f"Setup project.json for project {self.title} in {wd}/.VIS/")


        #Get VIS Root location
        self.p_vis = VIStk.__file__.replace("__init__.pyc","").replace("__init__.py","")

        self.p_project = getPath()
        self.p_vinfo = self.p_project + "/.VIS"
        self.p_sinfo = self.p_vinfo + "/project.json"
        with open(self.p_sinfo,"r") as f: 
            info = json.load(f)
            self.title = list(info.keys())[0]
            self.version = info[self.title]["metadata"]["version"]
            self.company = info[self.title]["metadata"]["company"]
            
        self.screenlist = []
        self.p_screens = self.p_project +"/Screens"
        self.p_modules = self.p_project +"/modules"
        self.p_templates = self.p_vinfo + "/Templates"
        self.p_icons = self.p_project + "/Icons"
        self.p_images = self.p_project + "/Images"
        
    def setVersion(self,version:str):
        """Sets a new project version
        """
        with open(self.p_sinfo,"r") as f: 
            info = json.load(f)

        info[self.title]["metadata"]["version"] = version
        self.version = version

        with open(self.p_sinfo,"w") as f:
            json.dump(info,f,indent=4)

    def restoreAll(self):
        """Undoes screen isolation"""
        with open(self.p_sinfo,"r") as f:
            info = json.load(f)

        for i in info[self.title]["Screens"]:
            if info[self.title]["Screens"][i]["release"] is None:
                info[self.title]["Screens"][i]["release"] = True

        with open(self.p_sinfo,"w") as f:
            json.dump(info,f,indent=4)


from tkinter import *
from VIStk.Objects._WindowGeometry import WindowGeometry
from VIStk.Objects._Window import Window

class SubRoot(Toplevel, Window):
    """A wrapper for the Toplevel class with VIS attributes"""
    def __init__(self,*args,**kwargs):
        super().__init__(*args,**kwargs)
        self.WindowGeometry = WindowGeometry(self)

    def modalize(self):
        """Makes the SubWindow modal"""
        self.focus_force()

        self.transient(self.master)
        self.grab_set()

        self.master.wait_window(self)


from VIStk.Widgets._MenuItem import MenuItem
from VIStk.Widgets._VISMenu import Menu
from VIStk.Widgets._QuestionWindow import QuestionWindow
from VIStk.Widgets._MenuWindow import MenuWindow
from VIStk.Widgets._WarningWindow import WarningWindow

__all__ = ["Menu","MenuItem","MenuWindow","QuestionWindow","WarningWindow"]